(self["webpackChunktine20_tinebase"] = self["webpackChunktine20_tinebase"] || []).push([[3077],{

/***/ 70439:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

"use strict";


const util = __webpack_require__(40971);

const DEFAULT_RULES = {};
let IS_DEFAULT_AUTO_TRIM = 0;

class BaseValidator {
    constructor(livrRules, isAutoTrim) {
        this.isPrepared = false;
        this.livrRules = livrRules;
        this.validators = {};
        this.validatorBuilders = {};

        if (isAutoTrim !== null && isAutoTrim !== undefined) {
            this.isAutoTrim = isAutoTrim;
        } else {
            this.isAutoTrim = IS_DEFAULT_AUTO_TRIM;
        }

        this.registerRules(DEFAULT_RULES);
    }

    static getDefaultRules() {
        return DEFAULT_RULES;
    }

    static registerAliasedDefaultRule(alias) {
        if (!alias.name) throw 'Alias name required';

        DEFAULT_RULES[alias.name] = this.buildAliasedRule(alias.rules, alias.error);
    }

    static registerDefaultRules(rules) {
        for (const ruleName in rules) {
            DEFAULT_RULES[ruleName] = rules[ruleName];
        }
    }

    static defaultAutoTrim(isAutoTrim) {
        IS_DEFAULT_AUTO_TRIM = !!isAutoTrim;
    }

    prepare() {
        const allRules = this.livrRules;

        for (const field in allRules) {
            let fieldRules = allRules[field];

            if (!Array.isArray(fieldRules)) {
                fieldRules = [fieldRules];
            }

            const validators = [];

            for (const fieldRule of fieldRules) {
                const parsed = this._parseRule(fieldRule);
                validators.push(this._buildValidator(parsed.name, parsed.args));
            }

            this.validators[field] = validators;
        }

        this.isPrepared = true;
        return this;
    }

    registerRules(rules) {
        for (const ruleName in rules) {
            this.validatorBuilders[ruleName] = rules[ruleName];
        }

        return this;
    }

    registerAliasedRule(alias) {
        if (!alias.name) throw 'Alias name required';

        this.validatorBuilders[alias.name] = this.constructor.buildAliasedRule(
            alias.rules,
            alias.error
        );

        return this;
    }

    getRules() {
        return this.validatorBuilders;
    }

    _parseRule(livrRule) {
        let name;
        let args;

        if (util.isObject(livrRule)) {
            name = Object.keys(livrRule)[0];
            args = livrRule[name];

            if (!Array.isArray(args)) args = [args];
        } else {
            name = livrRule;
            args = [];
        }

        return { name, args };
    }

    _buildValidator(name, args) {
        if (!this.validatorBuilders[name]) {
            throw 'Rule [' + name + '] not registered';
        }

        const allArgs = [];

        allArgs.push.apply(allArgs, args);
        allArgs.push(this.getRules());

        return this.validatorBuilders[name].apply(null, allArgs);
    }

    _autoTrim(data) {
        const dataType = typeof data;

        if (dataType !== 'object' && data) {
            if (data.replace) {
                return data.replace(/^\s*/, '').replace(/\s*$/, '');
            } else {
                return data;
            }
        } else if (dataType == 'object' && Array.isArray(data)) {
            const trimmedData = [];

            for (const item of data) {
                trimmedData.push(this._autoTrim(item));
            }

            return trimmedData;
        } else if (dataType == 'object' && util.isObject(data)) {
            const trimmedData = {};

            for (const key in data) {
                if (data.hasOwnProperty(key)) {
                    trimmedData[key] = this._autoTrim(data[key]);
                }
            }

            return trimmedData;
        }

        return data;
    }
}

module.exports = BaseValidator;


/***/ }),

/***/ 76600:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const Validator = __webpack_require__(57757);
const util = __webpack_require__(40971);

const rules = {
    required:         __webpack_require__(80926),
    not_empty:        __webpack_require__(28442),
    not_empty_list:   __webpack_require__(21673),
    any_object:       __webpack_require__(61865),

    string:           __webpack_require__(54272),
    eq:               __webpack_require__(21200),
    one_of:           __webpack_require__(88008),
    max_length:       __webpack_require__(98972),
    min_length:       __webpack_require__(85830),
    length_equal:     __webpack_require__(23410),
    length_between:   __webpack_require__(40013),
    like:             __webpack_require__(81319),

    integer:          __webpack_require__(90155),
    positive_integer: __webpack_require__(55842),
    decimal:          __webpack_require__(56939),
    positive_decimal: __webpack_require__(6927),
    max_number:       __webpack_require__(46386),
    min_number:       __webpack_require__(60019),
    number_between:   __webpack_require__(50862),

    email:            __webpack_require__(59847),
    equal_to_field:   __webpack_require__(83866),
    url:              __webpack_require__(4980),
    iso_date:         __webpack_require__(11178),

    default:          __webpack_require__(49656),
    trim:             __webpack_require__(95738),
    to_lc:            __webpack_require__(14450),
    to_uc:            __webpack_require__(18201),
    remove:           __webpack_require__(81534),
    leave_only:       __webpack_require__(10580),

    nested_object:    __webpack_require__(6055),
    variable_object:  __webpack_require__(23314),
    list_of:          __webpack_require__(65097),
    list_of_objects:  __webpack_require__(8518),
    or:               __webpack_require__(6635),
    list_of_different_objects: __webpack_require__(36072),
};

Validator.registerDefaultRules(rules);

module.exports = { Validator, rules, util };


/***/ }),

/***/ 57757:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

"use strict";


const BaseValidator = __webpack_require__(70439);
const util = __webpack_require__(40971);

const DEFAULT_RULES = {};
let IS_DEFAULT_AUTO_TRIM = 0;

class Validator extends BaseValidator {
    constructor(...args) {
        super(...args);
        this.errors = null;
    }

    static buildAliasedRule(rules, errorCode) {
        if (!rules) throw 'Alias rules required';

        const livr = { value: rules };

        return ruleBuilders => {
            // ruleBuilders comes always as a last argument
            // we register them to support custom rules in aliases
            const validator = new Validator(livr).registerRules(ruleBuilders).prepare();

            return (value, undefined, outputArr) => {
                const result = validator.validate({ value });

                if (result) {
                    outputArr.push(result.value);
                    return;
                } else {
                    return errorCode || validator.getErrors().value;
                }
            };
        };
    }

    validate(data) {
        if (!this.isPrepared) this.prepare();

        if (!util.isObject(data)) {
            this.errors = 'FORMAT_ERROR';
            return;
        }

        if (this.isAutoTrim) {
            data = this._autoTrim(data);
        }

        const errors = {};
        const result = {};

        for (const fieldName in this.validators) {
            const validators = this.validators[fieldName];
            if (!validators || !validators.length) continue;

            const value = data[fieldName];

            for (const validator of validators) {
                const fieldResultArr = [];

                const errCode = validator(
                    result.hasOwnProperty(fieldName) ? result[fieldName] : value,
                    data,
                    fieldResultArr
                );

                if (errCode) {
                    errors[fieldName] = errCode;
                    break;
                } else if (fieldResultArr.length) {
                    result[fieldName] = fieldResultArr[0];
                } else if (data.hasOwnProperty(fieldName) && !result.hasOwnProperty(fieldName)) {
                    result[fieldName] = value;
                }
            }
        }

        if (util.isEmptyObject(errors)) {
            this.errors = null;
            return result;
        } else {
            this.errors = errors;
            return false;
        }
    }

    getErrors() {
        return this.errors;
    }
}

module.exports = Validator;


/***/ }),

/***/ 61865:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function any_object() {
    return value => {
        if (util.isNoValue(value)) return;

        if (!util.isObject(value)) {
            return 'FORMAT_ERROR';
        }
    };
};

module.exports = any_object;

/***/ }),

/***/ 28442:
/***/ ((module) => {

function not_empty() {
    return value => {
        if (value !== null && value !== undefined && value === '') {
            return 'CANNOT_BE_EMPTY';
        }

        return;
    };
}

module.exports = not_empty;

/***/ }),

/***/ 21673:
/***/ ((module) => {

function not_empty_list() {
    return list => {
        if (list === undefined || list === '') return 'CANNOT_BE_EMPTY';
        if (!Array.isArray(list)) return 'FORMAT_ERROR';
        if (list.length < 1) return 'CANNOT_BE_EMPTY';
        return;
    };
};

module.exports = not_empty_list;

/***/ }),

/***/ 80926:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function required() {
    return value => {
        if (util.isNoValue(value)) {
            return 'REQUIRED';
        }

        return;
    };
};

module.exports = required;

/***/ }),

/***/ 65097:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const Validator = __webpack_require__(57757);
const util = __webpack_require__(40971);

function list_of(rules, ruleBuilders) {
    if (!Array.isArray(rules)) {
        rules = Array.prototype.slice.call(arguments);
        ruleBuilders = rules.pop();
    }

    const livr = { field: rules };
    const validator = new Validator(livr).registerRules(ruleBuilders).prepare();

    return (values, params, outputArr) => {
        if (util.isNoValue(values)) return;

        if (!Array.isArray(values)) return 'FORMAT_ERROR';

        const results = [];
        const errors = [];
        let hasErrors = false;

        for (const value of values) {
            const result = validator.validate({ field: value });

            if (result) {
                results.push(result.field);
                errors.push(null);
            } else {
                hasErrors = true;
                errors.push(validator.getErrors().field);
                results.push(null);
            }
        }

        if (hasErrors) {
            return errors;
        } else {
            outputArr.push(results);
            return;
        }
    };
}

module.exports = list_of;

/***/ }),

/***/ 36072:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const Validator = __webpack_require__(57757);
const util = __webpack_require__(40971);

function list_of_different_objects(selectorField, livrs, ruleBuilders) {
    const validators = {};

    for (const selectorValue in livrs) {
        const validator = new Validator(livrs[selectorValue])
            .registerRules(ruleBuilders)
            .prepare();
        validators[selectorValue] = validator;
    }

    return (objects, params, outputArr) => {
        if (util.isNoValue(objects)) return;
        if (!Array.isArray(objects)) return 'FORMAT_ERROR';

        const results = [];
        const errors = [];
        let hasErrors = false;

        for (const object of objects) {
            if (
                typeof object != 'object' ||
                !object[selectorField] ||
                !validators[object[selectorField]]
            ) {
                errors.push('FORMAT_ERROR');
                continue;
            }

            const validator = validators[object[selectorField]];
            const result = validator.validate(object);

            if (result) {
                results.push(result);
                errors.push(null);
            } else {
                hasErrors = true;
                errors.push(validator.getErrors());
                results.push(null);
            }
        }

        if (hasErrors) {
            return errors;
        } else {
            outputArr.push(results);
            return;
        }
    };
}

module.exports = list_of_different_objects;

/***/ }),

/***/ 8518:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const Validator = __webpack_require__(57757);
const util = __webpack_require__(40971);

function list_of_objects(livr, ruleBuilders) {
    const validator = new Validator(livr).registerRules(ruleBuilders).prepare();

    return (objects, params, outputArr) => {
        if (util.isNoValue(objects)) return;
        if (!Array.isArray(objects)) return 'FORMAT_ERROR';

        const results = [];
        const errors = [];
        let hasErrors = false;

        for (const object of objects) {
            const result = validator.validate(object);

            if (result) {
                results.push(result);
                errors.push(null);
            } else {
                hasErrors = true;
                errors.push(validator.getErrors());
                results.push(null);
            }
        }

        if (hasErrors) {
            return errors;
        } else {
            outputArr.push(results);
            return;
        }
    };
}

module.exports = list_of_objects;

/***/ }),

/***/ 6055:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const Validator = __webpack_require__(57757);
const util = __webpack_require__(40971);

function nested_object(livr, ruleBuilders) {
    const validator = new Validator(livr).registerRules(ruleBuilders).prepare();

    return (nestedObject, params, outputArr) => {
        if (util.isNoValue(nestedObject)) return;
        if (!util.isObject(nestedObject)) return 'FORMAT_ERROR';

        const result = validator.validate(nestedObject);

        if (result) {
            outputArr.push(result);
            return;
        } else {
            return validator.getErrors();
        }
    };
}

module.exports = nested_object;

/***/ }),

/***/ 6635:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const Validator = __webpack_require__(57757);

function or() {
    const ruleSets = Array.prototype.slice.call(arguments);
    const ruleBuilders = ruleSets.pop();

    const validators = ruleSets.map(rules => {
        const livr = { field: rules };
        const validator = new Validator(livr).registerRules(ruleBuilders).prepare();

        return validator;
    });

    return (value, params, outputArr) => {
        let lastError;

        for (const validator of validators) {
            const result = validator.validate({ field: value });

            if (result) {
                outputArr.push(result.field);
                return;
            } else {
                lastError = validator.getErrors().field;
            }
        }

        return lastError;
    };
}

module.exports = or;

/***/ }),

/***/ 23314:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const Validator = __webpack_require__(57757);
const util = __webpack_require__(40971);

function variable_object(selectorField, livrs, ruleBuilders) {
    const validators = {};

    for (const selectorValue in livrs) {
        const validator = new Validator(livrs[selectorValue])
            .registerRules(ruleBuilders)
            .prepare();
        validators[selectorValue] = validator;
    }

    return (object, params, outputArr) => {
        if (util.isNoValue(object)) return;

        if (
            !util.isObject(object) ||
            !object[selectorField] ||
            !validators[object[selectorField]]
        ) {
            return 'FORMAT_ERROR';
        }

        const validator = validators[object[selectorField]];
        const result = validator.validate(object);

        if (result) {
            outputArr.push(result);
            return;
        } else {
            return validator.getErrors();
        }
    };
}

module.exports = variable_object;


/***/ }),

/***/ 49656:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

module.exports = defaultValue => {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) {
            outputArr.push(defaultValue);
        }
    };
}




/***/ }),

/***/ 10580:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function leave_only(chars) {
    chars = util.escapeRegExp(chars);
    const re = new RegExp('[^' + chars + ']', 'g');

    return (value, params, outputArr) => {
        if (util.isNoValue(value) || typeof value === 'object') return;

        value += ''; // TODO just skip numbers
        outputArr.push(value.replace(re, ''));
    };
}

module.exports = leave_only;

/***/ }),

/***/ 81534:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function remove(chars) {
    chars = util.escapeRegExp(chars);
    const re = new RegExp('[' + chars + ']', 'g');

    return (value, params, outputArr) => {
        if (util.isNoValue(value) || typeof value === 'object') return;

        value += ''; // TODO just skip numbers
        outputArr.push(value.replace(re, ''));
    };
}

module.exports = remove;

/***/ }),

/***/ 14450:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function to_lc() {
    return (value, params, outputArr) => {
        if (util.isNoValue(value) || typeof value === 'object') return;

        value += ''; // TODO just skip numbers
        outputArr.push(value.toLowerCase());
    };
}

module.exports = to_lc;

/***/ }),

/***/ 18201:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function to_uc() {
    return (value, params, outputArr) => {
        if (util.isNoValue(value) || typeof value === 'object') return;

        value += ''; // TODO just skip numbers
        outputArr.push(value.toUpperCase());
    };
}

module.exports = to_uc;

/***/ }),

/***/ 95738:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function trim() {
    return (value, params, outputArr) => {
        if (util.isNoValue(value) || typeof value === 'object') return;

        value += ''; // TODO just do not trim numbers
        outputArr.push(value.replace(/^\s*/, '').replace(/\s*$/, ''));
    };
}

module.exports = trim;

/***/ }),

/***/ 56939:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function decimal() {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';
        if (!util.looksLikeNumber(value)) return 'NOT_DECIMAL';

        value += '';
        if (!/^(?:\-?(?:(?:[0-9]+\.[0-9]+)|(?:[0-9]+)))$/.test(value)) return 'NOT_DECIMAL';
        outputArr.push(+value);
    };
}

module.exports = decimal;

/***/ }),

/***/ 90155:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function integer() {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';
        if (!util.looksLikeNumber(value)) return 'NOT_INTEGER';

        if (!Number.isInteger(+value)) return 'NOT_INTEGER';
        outputArr.push(+value);
    };
}

module.exports = integer;

/***/ }),

/***/ 46386:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function max_number(maxNumber) {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';
        if (!util.looksLikeNumber(value)) return 'NOT_NUMBER';

        if (+value > +maxNumber) return 'TOO_HIGH';
        outputArr.push(+value);
    };
}

module.exports = max_number;

/***/ }),

/***/ 60019:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function min_number(minNumber) {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';
        if (!util.looksLikeNumber(value)) return 'NOT_NUMBER';

        if (+value < +minNumber) return 'TOO_LOW';
        outputArr.push(+value);
    };
}

module.exports = min_number;

/***/ }),

/***/ 50862:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function number_between(minNumber, maxNumber) {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';
        if (!util.looksLikeNumber(value)) return 'NOT_NUMBER';

        if (+value < +minNumber) return 'TOO_LOW';
        if (+value > +maxNumber) return 'TOO_HIGH';
        outputArr.push(+value);
    };
}

module.exports = number_between;

/***/ }),

/***/ 6927:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function positive_decimal() {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';
        if (!util.looksLikeNumber(value)) return 'NOT_POSITIVE_DECIMAL';

        if (Number.isNaN(+value) || +value <= 0) return 'NOT_POSITIVE_DECIMAL';
        outputArr.push(+value);
    };
}

module.exports = positive_decimal;


/***/ }),

/***/ 55842:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function positive_integer() {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';
        if (!util.looksLikeNumber(value)) return 'NOT_POSITIVE_INTEGER';

        if (!Number.isInteger(+value) || +value < 1) return 'NOT_POSITIVE_INTEGER';
        outputArr.push(+value);
    };
}

module.exports = positive_integer;

/***/ }),

/***/ 59847:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function email() {
    var emailRe = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

    return value => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        value += '';
        if (!emailRe.test(value)) return 'WRONG_EMAIL';
        if (/\@.*\@/.test(value)) return 'WRONG_EMAIL';
        if (/\@.*_/.test(value)) return 'WRONG_EMAIL';
        return;
    };
}

module.exports = email;

/***/ }),

/***/ 83866:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function equal_to_field(field) {
    return (value, params) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        if (value != params[field]) return 'FIELDS_NOT_EQUAL';
        return;
    };
}

module.exports = equal_to_field;

/***/ }),

/***/ 11178:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function iso_date() {
    return value => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        const matched = value.match(/^(\d{4})-([0-1][0-9])-([0-3][0-9])$/);

        if (matched) {
            const epoch = Date.parse(value);
            if (!epoch && epoch !== 0) return 'WRONG_DATE';

            const d = new Date(epoch);
            d.setTime(d.getTime() + d.getTimezoneOffset() * 60 * 1000);

            if (
                d.getFullYear() == matched[1] &&
                d.getMonth() + 1 == +matched[2] &&
                d.getDate() == +matched[3]
            ) {
                return;
            }
        }

        return 'WRONG_DATE';
    };
}

module.exports = iso_date;

/***/ }),

/***/ 4980:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function url() {
    const urlReStr =
        '^(?:(?:http|https)://)(?:\\S+(?::\\S*)?@)?(?:(?:(?:[1-9]\\d?|1\\d\\d|2[0-1]\\d|22[0-3])(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}(?:\\.(?:[0-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))|(?:(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)(?:\\.(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)*(?:\\.(?:[a-z\\u00a1-\\uffff]{2,})))\\.?|localhost)(?::\\d{2,5})?(?:[/?#]\\S*)?$';
    const urlRe = new RegExp(urlReStr, 'i');

    return value => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        if (value.length < 2083 && urlRe.test(value)) return;
        return 'WRONG_URL';
    };
}

module.exports = url;

/***/ }),

/***/ 21200:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function eq(allowedValue) {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        if (value + '' === allowedValue + '') {
            outputArr.push(allowedValue);
            return;
        }

        return 'NOT_ALLOWED_VALUE';
    };
}

module.exports = eq;

/***/ }),

/***/ 40013:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function length_between(minLength, maxLength) {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        value += '';
        if (value.length < minLength) return 'TOO_SHORT';
        if (value.length > maxLength) return 'TOO_LONG';
        outputArr.push(value);
    };
}

module.exports = length_between;

/***/ }),

/***/ 23410:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function length_equal(length) {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        value += '';
        if (value.length < length) return 'TOO_SHORT';
        if (value.length > length) return 'TOO_LONG';
        outputArr.push(value);
    };
}

module.exports = length_equal;

/***/ }),

/***/ 81319:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function like(reStr, flags) {
    const isIgnoreCase = arguments.length === 3 && flags.match('i');
    const re = new RegExp(reStr, isIgnoreCase ? 'i' : '');

    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;

        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        value += '';
        if (!value.match(re)) return 'WRONG_FORMAT';
        outputArr.push(value);
    };
}

module.exports = like;

/***/ }),

/***/ 98972:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);
 
function max_length(maxLength) {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        value += '';
        if (value.length > maxLength) return 'TOO_LONG';
        outputArr.push(value);
    };
}

module.exports = max_length;

/***/ }),

/***/ 85830:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function min_length(minLength) {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        value += '';
        if (value.length < minLength) return 'TOO_SHORT';
        outputArr.push(value);
    };
}

module.exports = min_length;

/***/ }),

/***/ 88008:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function one_of(allowedValues) {
    if (!Array.isArray(allowedValues)) {
        allowedValues = Array.prototype.slice.call(arguments);
        allowedValues.pop(); // pop ruleBuilders
    }

    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;

        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        for (const allowedValue of allowedValues) {
            if (value + '' === allowedValue + '') {
                outputArr.push(allowedValue);
                return;
            }
        }

        return 'NOT_ALLOWED_VALUE';
    };
}

module.exports = one_of;

/***/ }),

/***/ 54272:
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

const util = __webpack_require__(40971);

function string() {
    return (value, params, outputArr) => {
        if (util.isNoValue(value)) return;
        if (!util.isPrimitiveValue(value)) return 'FORMAT_ERROR';

        outputArr.push(value + '');
        return;
    };
}

module.exports = string;

/***/ }),

/***/ 40971:
/***/ ((module) => {

module.exports = {
    isPrimitiveValue(value) {
        if (typeof value == 'string') return true;
        if (typeof value == 'number' && isFinite(value)) return true;
        if (typeof value == 'boolean') return true;
        return false;
    },

    looksLikeNumber(value) {
        if (!isNaN(+value)) return true;
        return false;
    },

    isObject(obj) {
        return Object(obj) === obj && Object.getPrototypeOf(obj) === Object.prototype;
    },

    isEmptyObject(map) {
        for (const key in map) {
            if (map.hasOwnProperty(key)) {
                return false;
            }
        }
        return true;
    },

    escapeRegExp(str) {
        return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, '\\$&');
    },

    isNoValue(value) {
        return value === undefined || value === null || value === '';
    }
};


/***/ })

}]);